#ifndef __PIPE_H
#define __PIPE_H

#include "types.h"
#include "spinlock.h"
#include "file.h"

#define PIPESIZE 512
#define PIPE_TIMEOUT_TICKS 1000
#define PIPE_MAX_RETRIES 3

typedef enum {
    PIPE_OK = 0,
    PIPE_ERROR = -1,
    PIPE_TIMEOUT = -2,
    PIPE_CLOSED = -3,
    PIPE_INVALID_PARAM = -4,
    PIPE_BUFFER_FULL = -5
} pipe_status_t;

struct pipe {
    struct spinlock lock;
    char data[PIPESIZE];
    uint nread;      // number of bytes read
    uint nwrite;     // number of bytes written
    int readopen;    // read fd is still open
    int writeopen;   // write fd is still open
    uint64 last_op; // timestamp of last operation
    int error;      // last error code
};

pipe_status_t pipealloc(struct file **f0, struct file **f1);
pipe_status_t pipeclose(struct pipe *pi, int writable);
pipe_status_t pipewrite(struct pipe *pi, uint64 addr, int n);
pipe_status_t piperead(struct pipe *pi, uint64 addr, int n);

#endif