#include "kernel/include/types.h"
#include "kernel/include/stat.h"
#include "user.h"
#define PGSIZE 4096
void test_alloc()
{
  void *p1, *p2, *p3, *p4;

  // 测试1：基本分配和释放
  printf("Test1: Basic allocation\n");
  p1 = sbrk(4096);
  printf("Allocated page at %p\n", p1);
  sbrk(-4096);

  // 测试2：多页分配
  printf("Test2: Multiple pages\n");
  p1 = sbrk(8192);
  p2 = sbrk(4096);
  printf("Allocated pages at %p and %p\n", p1, p2);
  sbrk(-12288);

  // 测试3：碎片化测试
  printf("Test3: Fragmentation test\n");
  p1 = sbrk(4096);
  p2 = sbrk(4096);
  p3 = sbrk(4096);
  printf("Allocated 3 pages at %p, %p, %p\n", p1, p2, p3);
  sbrk(-4096);
  p4 = sbrk(4096);
  printf("Reallocated page at %p\n", p4);

  // 释放所有内存
  sbrk(-12288);

  // 测试4：大块内存分配
  printf("Test4: Large allocation\n");
  void *large = sbrk(16384); // 4页
  printf("Large block at %p\n", large);
  sbrk(-16384);

  // 测试5：边界测试
  printf("Test5: Boundary test\n");
  void *max_alloc = sbrk(1048576); // 256页
  printf("Max allocation at %p\n", max_alloc);
  if (max_alloc != (void *)-1)
  {
    sbrk(-1048576);
  }

  printf("Tests completed\n");
}

void test_buddy_specific()
{
  printf("Test6: Buddy System Specific\n");

  // 基本页面分配测试
  void *p1 = sbrk(PGSIZE);
  if (p1 == (void *)-1)
  {
    printf("Basic allocation failed\n");
    return;
  }
  printf("Allocated page at %p\n", p1);

  // 验证页面对齐
  if ((uint64)p1 % PGSIZE == 0)
  {
    printf("Page alignment verified\n");
  }

  // 分配第二个页面测试伙伴
  void *p2 = sbrk(PGSIZE);
  if (p2 != (void *)-1)
  {
    printf("Buddy page at %p\n", p2);
  }

  // 安全释放顺序
  if (p2 != (void *)-1)
  {
    sbrk(-PGSIZE); // 释放p2
    printf("Buddy page freed\n");
  }
  sbrk(-PGSIZE); // 释放p1
  printf("Test completed successfully\n");
}

int main()
{
  test_alloc();
  test_buddy_specific();
  exit(0);
}